library ieee;
use ieee.std_logic_1164.all;

use work.types.all;
use work.components.all;
use work.CAMComponents.cellacc_toplevel;

entity cellacc_IFM is port (
    i_clk : in STD_LOGIC;
    i_rst : in STD_LOGIC;
    i_start : in STD_LOGIC;
    i_opt : in STD_LOGIC_VECTOR(31 downto 0);
    o_rdy : out STD_LOGIC;
    acc_in_0 : in STD_LOGIC_VECTOR(31 downto 0);
	acc_in_1 : in STD_LOGIC_VECTOR(31 downto 0);
    acc_out_0 : out STD_LOGIC_VECTOR(31 downto 0);
	acc_out_1 : out STD_LOGIC_VECTOR(31 downto 0)
);
end cellacc_IFM;

architecture rtl of cellacc_IFM is
	constant ca_cells_per_sub : integer := 10;
	constant ca_num_grids : integer := 4;
	constant ca_num_neighbours : integer := 8;
	constant ca_height : integer := 48;
begin
	-- GENERICS: CellsPerSub, NumGrids, NumNeighbours, Height
	cell_acc : cellacc_toplevel generic map(ca_cells_per_sub, ca_num_grids, ca_num_neighbours, ca_height) port map (
		i_start => i_start,
		i_reset => i_rst,
		i_clk => i_clk,
		i_opcode => i_opt(31 downto 29),
		i_operand => i_opt(28 downto 0),
		-- If less than 33 bits:
--		i_operand2(ca_height - 1 downto 0) => acc_in_0, -- Note, range needs to be height - 1 downto 0
		-- Else comment the above line, and uncomment this line
		i_operand2(31 downto 0) => acc_in_0, -- Note, range needs to be height - 1 downto 0
		-- If more than 32 bits, use additional assignment from acc_in_1, if more than 64,
		-- then additional ports must be set up in a25_coprocessor_cellacc.v
		-- By convention, highest bits go in the highest numbered input-reg. (i.e. 0-31 -> acc_in_0, 32-63 -> acc_in_1)
		i_operand2(ca_height - 1 downto 32) => acc_in_1(ca_height - 33 downto 0), -- Uncomment and adjust for > 32 height.
		o_done => o_rdy,
		-- There is always atleast 32 bits of output
		o_output => acc_out_0,
		o_output2(ca_height - 33 downto 0) => acc_out_1(ca_height - 33 downto 0)
	);
	-- Define the unused output fields as 0. (This should be the highest USED output, as well as any surplus outputs.
	acc_out_1(31 downto ca_height - 33 + 1) <= (OTHERS => '0');
end rtl;
