----------------------------------------------------------------------------------
-- Company: Norges Teknisk Naturvitenskapelige Universitet
-- Engineer: Einar Johan Trøan Sømåen
-- 
-- Module Name:    three_cell_ent - Behavioral 
-- Project Name: Cellular Automata Accelerator for SHMAC
----------------------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all;

LIBRARY std;
USE std.standard.all;

--! A single cell

ENTITY three_cell_ent IS
	GENERIC (num_neighbours : INTEGER := 8);
	PORT (
		i_neighbour_states : IN STD_LOGIC_VECTOR(num_neighbours - 1 DOWNTO 0); --! Connections to neighbours
		o_query : OUT STD_LOGIC_VECTOR (num_neighbours DOWNTO 0); --! Values sent to rule-unit
		i_rule_back : IN STD_LOGIC; --! Result from rule-unit
		i_update : IN STD_LOGIC; --! Trigger for updating from rule unit (if 1 then store)
		i_clk : IN STD_LOGIC
	);
END three_cell_ent;

ARCHITECTURE three_cell OF three_cell_ent IS
BEGIN
	-- Connect the neighbour's states to the query from this cell.
	o_query(num_neighbours/2 - 1 downto 0) <= i_neighbour_states(num_neighbours/2 - 1 downto 0);
	o_query(num_neighbours downto num_neighbours/2 + 1) <= i_neighbour_states(num_neighbours - 1 downto num_neighbours/2);

	update_by_rule: PROCESS(i_clk) 
	BEGIN
		IF (rising_edge(i_clk)) THEN
			IF (i_update = '1') THEN
				o_query(num_neighbours/2) <= i_rule_back;
			END IF;
		END IF;
	END PROCESS;
END three_cell;
