#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "random.h"
#include "cam.h"

#define RULE_SIZE 64 // Number of bytes to describe a rule.

typedef struct {
	unsigned char _rule[RULE_SIZE];
	float _fitness;
} genome_info;

char *g_grid_temp = NULL;
char *g_fitness_data = NULL;

float get_random_float() {
	unsigned int max_uint = 0xFFFFFFFF;
	return rng_get_uint() / (float) max_uint;
}

void init_genome(genome_info *genome) {
	int neighbours = cam_get_num_neighbours();
	for (int i = 0; i < (1 << (neighbours - 2)); i++) {
		genome->_rule[i] = rng_get_byte();
	}
	genome->_fitness = 0.0f;
}

void init_population(genome_info *population, int count) {
	for (int i = 0; i < count; i++) {
		init_genome(population + i);
	}
}
// Traffic:
void init_fitness(char *init_val) {
	free(g_fitness_data);
	int width = cam_get_width();
	int height = cam_get_height();

	// This function is called once at the start of each population test.
	// Write any precomputation code here (i.e. perform density classification in software)
	// g_fitness_data can then be used in the fitness-function afterwards.
	
	g_fitness_data = NULL;
}

float fitness() {
	unsigned int width = cam_get_width();
	unsigned int height = cam_get_height();
	unsigned int total = width * height;

	cam_get_full_grid(g_grid_temp);

	unsigned int count = 0;
	char last_char = '0';
	unsigned int correct_rows = 0;
	int first_pos = -1;
	int numOnFirstPos = 0;
	int num_groups = 0;
	for(int y = 0; y < height; y++) {

		for(int x = 0; x < width; x++) {
			if (g_grid_temp[x * height + y] == '1') {
				count++;
			}
		}
	}
	unsigned int size = width * height;
	float result = count/(float)size;

	return result;
}

void evaluate_genome(genome_info *genome, unsigned int iterations, const char *init_val) {
	cam_reset_to(0);
	cam_set_rule_from_array(genome->_rule);
	cam_load_full_grid(init_val);
	cam_set_edge(1, 1, 0);
	cam_run(iterations);
	genome->_fitness += fitness();
}

void evaluate_pop(genome_info *population, int count, unsigned int iterations, const char *init_val) {
	for (int i = 0; i < count; i++) {
		evaluate_genome(population + i, iterations, init_val);
	}
}

// Implicitly one grid-size
void initialize_with_random(char *dst) {
	unsigned int width = cam_get_width();
	unsigned int height = cam_get_height();

	unsigned int bytes_left = width * height;
	unsigned int index = 0;
	while (bytes_left > 0) {
		unsigned int val = rng_get_byte();
		if (bytes_left < 8) {
			hex2bin(val, dst + index, bytes_left, 0);
			bytes_left = 0;
		} else {
			hex2bin(val, dst + index, 8, 0);
			bytes_left -= 8;
			index += 8;
		}
	}	
}

genome_info *select(genome_info *population, unsigned int pop_size, float prob) {
	genome_info *selected = (genome_info*)malloc(sizeof(genome_info) * pop_size);
	// Normalize: http://en.wikipedia.org/wiki/Selection_(genetic_algorithm)
	float fitness_sum = 0.0f;
	for (int i = 0; i < pop_size; i++) {
		fitness_sum += population[i]._fitness;
	}

	for (int i = 0; i < pop_size; i++) {
		population[i]._fitness /= fitness_sum;
	}
	
	fitness_sum = 0.0f;

	for (int i = 0; i < pop_size; i++) {
		float temp = population[i]._fitness;
		population[i]._fitness += fitness_sum;
		fitness_sum += temp;
	}
	// Introduce two new genomes.
	init_genome(selected);
	init_genome(selected + pop_size - 3);
	// Retain the top two genomes:
	selected[pop_size - 1] = population[pop_size - 1];
	selected[pop_size - 2] = population[pop_size - 2];
	for (int i = 1; i < pop_size - 3; i++) {
		float random = get_random_float();
		for (int j = 0; j < pop_size; j++) {
			if (population[j]._fitness >= random) {
				selected[i] = population[j];
				break;
			}
		}
	}
	printf("Select done\n");
	return selected;
}

// This function gets direct pointers to the last generation in, and should create new genomes.
genome_info *crossover(genome_info *population, unsigned int pop_size, float prob) {
	genome_info *result = (genome_info*)malloc(sizeof(genome_info) * pop_size);
	for (int i = 0; i < pop_size; i += 2) {
		float random = get_random_float();
		if (random <= prob) {
			unsigned int split_pos = rng_get_uint() % (RULE_SIZE * 8);
			unsigned int split_point = split_pos / 8;
			unsigned int split_bit_pos = split_pos % 8;
			unsigned int split_mask = (1 << split_bit_pos) - 1;
			unsigned int split_mask2 = split_mask ^ 0xFFFFFFFF;
			for (int j = 0; j < RULE_SIZE; j++) {
				if (j < split_point) {
					result[i]._rule[j] = population[i]._rule[j];
					result[i + 1]._rule[j] = population[i + 1]._rule[j];
				} else if (j > split_point) {
					result[i]._rule[j] = population[i + 1]._rule[j];
					result[i + 1]._rule[j] = population[i]._rule[j];
				} else {
					unsigned int split_byte = (population[i]._rule[split_point] & split_mask2) & (population[i + 1]._rule[split_point] & split_mask);
					unsigned int split_byte2 = (population[i + 1]._rule[split_point] & split_mask2) & (population[i]._rule[split_point] & split_mask);
					result[i]._rule[j] = split_byte;
					result[i + 1]._rule[j] = split_byte2;
				}
			}
		} else {
			memcpy(result[i]._rule, population[i]._rule, RULE_SIZE);
			memcpy(result[i + 1]._rule, population[i + 1]._rule, RULE_SIZE);
		}
		result[i]._fitness = 0.0f;
		result[i + 1]._fitness = 0.0f;
	}
	return result;
}

void mutate(genome_info *population, unsigned int pop_size, float prob) {
	for (int i = 0; i < pop_size; i++) {
		float random = get_random_float();
		if (random <= prob) {
			unsigned int byte_pos = rng_get_byte() % RULE_SIZE;
			unsigned int bit = rng_get_byte() % 8;
			population[i]._rule[byte_pos] = population[i]._rule[byte_pos] ^ (1 << bit);
		}
	}
}

void sort_pop(genome_info *population, unsigned int pop_size) {
	genome_info temp;
	for (int i = 1; i < pop_size; i++) {
		if (population[i]._fitness < population[i - 1]._fitness) {
			temp = population[i];
			for (int j = i; j >= 0; j--) {
				if (j == 0) {
					population[0] = temp;
					break;
				}
				if (population[j - 1]._fitness < temp._fitness) {
					population[j] = temp;
					break;
				}
				population[j] = population[j - 1];
			}
		}
	}
}

void print_fitness_values(genome_info *pop, int pop_size, int also_print_rules) {
	for (int j = 0; j < pop_size; j++) {
		printf("Genome %d : ", j);
		if (also_print_rules) {
			for (int i = 0; i < 64; i++) {
				printf("%02X", pop[j]._rule[i]);
			}
		}
		printf(" has fitness %f\n", pop[j]._fitness);
	}
}

void genetic() {
	unsigned int pop_size = 256;
	unsigned int num_generations = 10000;
	unsigned int num_iterations = 1000;

	float select_prob =  0.7;
	float crossover_prob = 0.6;
	float mutate_prob = 0.1;

	g_grid_temp = cam_allocate_grid_array();
	genome_info *pop = (genome_info*)malloc(sizeof(genome_info) * pop_size);

	char *init_val = cam_allocate_grid_array();
	initialize_with_random(init_val);
	init_fitness(init_val);
	float baseline = fitness();	
	printf("Baseline: %f\n", baseline);

	cam_load_full_grid(init_val);
	printf("\n");
	cam_set_edge(1, 1, 0);
	cam_set_wolfram_rule(184);
	cam_run(100);
	cam_set_wolfram_rule(232);
	cam_run(100);
	printf("Comparison with 184/232: %f\n", fitness());
	
	init_population(pop, pop_size);

	for (int x = 0; x < 10; x++) {
		printf("Sub-iteration: %d\n", x);
		evaluate_pop(pop, pop_size, num_iterations, init_val);
		initialize_with_random(init_val);
		init_fitness(init_val);
	}
	print_fitness_values(pop, pop_size, 1);
	sort_pop(pop, pop_size);
	print_fitness_values(pop, pop_size, 1);
	for (int i = 0; i < num_generations; i++) {
		genome_info *selected = select(pop, pop_size, select_prob);
		free(pop);
		genome_info *crossovered = crossover(selected, pop_size, crossover_prob);
		mutate(crossovered, pop_size, mutate_prob);
		free(selected);
		pop = crossovered;
		for (int x = 0; x < 5; x++) {
			printf("Sub-iteration: %d\n", x);
			evaluate_pop(pop, pop_size, num_iterations, init_val);
			initialize_with_random(init_val);
			init_fitness(init_val);
		}
		sort_pop(pop, pop_size); // According to http://en.wikipedia.org/wiki/Selection_(genetic_algorithm)
		printf("Generation %d\n", i + 1);
		print_fitness_values(pop, pop_size, 1);
	}

	free(g_grid_temp);
}
