#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "shmac/shmac.h"
#include "cam.h"
#include "tests.h"
#include "random.h"

// Just a function to waste some cycles, to avoid
// overflowing printf's buffer. 
float waste_cycles(int iterations) {
	float x = 1.0f;
	for (int i = 0; i < (iterations * 10); i++) {
		x += (x * i) / 1.01f;
	}
	return x;
}

int main(void) {
	*IC0_IRQ_ENABLESET = INT_MASK_HOST;
	setvbuf(stdout, NULL, _IONBF, 0);

	printf("*** Program start ***\n");
	cam_init();
	cam_reset_to(0);
	test_reset_to_0();
	test_reset_to_1();
	test_random_rule_30(200);
	test_wolfram_rules_multigen();
	test_wolfram_rules(cam_get_width());
	test_density_184_232_n_times(1000);
	
	// Examples of using the write-masks:
	printf("Test 1: Lock one corner.\n");
	cam_reset_to(0);
	cam_set_mask(0, 0, 0, 0, 0);
	cam_print_grid_rotated();
	cam_set_wolfram_rule(255);
	cam_run(1);
	printf("\n");
	cam_print_grid_rotated();

	printf("Verify that resetting still works\n");
	cam_reset_to(1);
	cam_print_grid_rotated(); printf("\n");

	printf("Test 2: Lock one column.\n");
	cam_reset_to(0);
	cam_set_mask(0, 1, 0, 0, 0);
	cam_print_grid_rotated();
	cam_set_wolfram_rule(255);
	printf("\n");
	cam_run(1);
	cam_print_grid_rotated();

	printf("Verify that resetting still works\n");
	cam_reset_to(1);
	cam_print_grid_rotated(); printf("\n");

	printf("Test 3: Lock one row.\n");
	cam_reset_to(0);
	cam_set_mask(1, 0, 0, 0, 0);
	cam_print_grid_rotated();
	cam_set_wolfram_rule(255);
	printf("\n");
	cam_run(1);
	cam_print_grid_rotated();

	printf("Verify that resetting still works\n");
	cam_reset_to(1);
	cam_print_grid_rotated(); printf("\n");

	printf("Test 4: Lock one row and column.\n");
	cam_reset_to(0);
	cam_set_mask(1, 1, 0, 0, 0);
	cam_print_grid_rotated();
	cam_set_wolfram_rule(255);
	printf("\n");
	cam_run(1);
	cam_print_grid_rotated();

	printf("Verify that resetting still works\n");
	cam_reset_to(1);
	cam_print_grid_rotated(); printf("\n");

	printf("Test 5: Lock one grid other than 0,0.\n");
	cam_reset_to(0);
	cam_set_mask(0, 0, 0, 3, 3);
	cam_print_grid_rotated();
	cam_set_wolfram_rule(255);
	printf("\n");
	cam_run(1);
	cam_print_grid_rotated();

	printf("Test 6: Unlock\n");
	cam_set_mask(0, 0, 1, 3, 3);
	cam_set_wolfram_rule(255);
	cam_run(1);
	cam_print_grid_rotated();

	return 0;
}
