#include <stdlib.h>
#include <stdio.h>
#include "random.h"
#include "cam.h"

#define RNG_BUFFER_SIZE 4096

unsigned char g_rng_buffer[RNG_BUFFER_SIZE];
unsigned int g_rng_buffer_index = 0;
char *g_rng_grid = NULL;

void rng_init(unsigned int seed) {
	cam_set_wolfram_rule(30);
	if (g_rng_grid == NULL) {
		cam_load_single_north_east_1();
		cam_create_multi_gen();
		int width = cam_get_width();
		int height = cam_get_height();
		g_rng_grid = (char*)malloc(sizeof(char) * width * height + 1);
		g_rng_grid[width * height] = '\0';
		cam_set_edge(1, 1, 0);
		cam_run(seed);
		for (int i = 0; i < RNG_BUFFER_SIZE; i++) {
			cam_run(8);
			g_rng_buffer[i] = (unsigned char)(cam_read_output(0) & 0xFF);
		}
		g_rng_buffer_index = 0;
		cam_get_full_grid(g_rng_grid);
	} else {
		cam_load_full_grid(g_rng_grid);
	}
	cam_set_edge(1, 1, 0);
}

void rng_free() {
	free(g_rng_grid);
	g_rng_grid = NULL;
	g_rng_buffer_index = 0;
}

void rng_seed(unsigned int seed) {
	rng_free();
	rng_init(seed);
}

void rng_fill_buffer() {
	// Assure that the CA is set up correctly
	int doUpdate = 1;
	if (g_rng_grid == NULL) {
		doUpdate = 0;
	}
	rng_init(0); // Will not reseed if there already exists data, otherwise it will.
	cam_set_edge(1, 1, 0);
	// If the grid was uninitialized when we started, then rng_init already filled the buffer
	if (doUpdate == 1) {
		for (int i = 0; i < RNG_BUFFER_SIZE; i++) {
			cam_run(8);
			g_rng_buffer[i] = (unsigned char)(cam_read_output(0) & 0xFF);
		}
		cam_get_full_grid(g_rng_grid);
		g_rng_buffer_index = 0;
	}
}

unsigned char rng_get_byte() {
	if (g_rng_buffer_index == RNG_BUFFER_SIZE || g_rng_grid == NULL) {
		rng_fill_buffer();
	}
	return g_rng_buffer[g_rng_buffer_index++];
}

unsigned int rng_get_uint() {
	unsigned int result;
	unsigned char *ptr = (unsigned char*)&result;
	ptr[0] = rng_get_byte();
	ptr[1] = rng_get_byte();
	ptr[2] = rng_get_byte();
	ptr[3] = rng_get_byte();
	return result;
}

void rng_get_bytes(unsigned int num_bytes, unsigned char *dst) {
	for (int i = 0; i < num_bytes; i++) {
		dst[i] = rng_get_byte();
	}
}
