//////////////////////////////////////////////////////////////////
//                                                              //
//  Co-processor module for Amber 25 Core                       //
//                                                              //
//  This file is part of the Amber project                      //
//  http://www.opencores.org/project,amber                      //
//                                                              //
//  Description                                                 //
//  Co_processor 15 registers and control signals               //
//  Author(s):                                                  //
//      - Conor Santifort, csantifort.amber@gmail.com           //
//                                                              //
//////////////////////////////////////////////////////////////////
//                                                              //
// Copyright (C) 2011 Authors and OPENCORES.ORG                 //
//                                                              //
// This source file may be used and distributed without         //
// restriction provided that this copyright statement is not    //
// removed from the file and that any derivative work contains  //
// the original copyright notice and the associated disclaimer. //
//                                                              //
// This source file is free software; you can redistribute it   //
// and/or modify it under the terms of the GNU Lesser General   //
// Public License as published by the Free Software Foundation; //
// either version 2.1 of the License, or (at your option) any   //
// later version.                                               //
//                                                              //
// This source is distributed in the hope that it will be       //
// useful, but WITHOUT ANY WARRANTY; without even the implied   //
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      //
// PURPOSE.  See the GNU Lesser General Public License for more //
// details.                                                     //
//                                                              //
// You should have received a copy of the GNU Lesser General    //
// Public License along with this source; if not, download it   //
// from http://www.opencores.org/lgpl.shtml                     //
//                                                              //
//////////////////////////////////////////////////////////////////

//////////////////////
//////WARNING!!///////
//////////////////////
//
//This file is used as a source file for accelerator interface tile generation!
//It contains tags used for automatic navigation by the generator script.
//make sure any changes made to this file is compatible with the rest of the system
//Files that can cause compatibility issues are listed below 
// /shmac/hardware/tileGenerator/tileGenerator.py
// /shmac/hardware/tileGenerator/templates/coproc_TEMP.v
// /shmac/hardware/units/amber/hw/vlog/amber25/a25_core.v
//
// Marton Teilgård mteilgard@gmail.com
//////////////////////

//Script generated defines 
`define ACC_INPUTS		2
`define ACC_OUTPUTS		2

module a25_coprocessor_cellacc
(
input                       i_clk,
input                       i_rst,
input                       i_core_stall,     // stall all stages of the Amber core at the same time
input       [2:0]           i_copro_opcode1,
input       [2:0]           i_copro_opcode2,
input       [3:0]           i_copro_crn,      // Register Number 
input       [3:0]           i_copro_crm,
input       [3:0]           i_copro_num,
input       [1:0]           i_copro_operation,
input       [31:0]          i_copro_write_data,

input                       i_fault,          // high to latch the fault address and status
input       [7:0]           i_fault_status,
input       [31:0]          i_fault_address,  // the address that caused the fault

output reg  [31:0]          o_copro_read_data,
output                      o_cache_enable,
output                      o_cache_flush,
output      [31:0]          o_cacheable_area 
);

// Bit 0 - Cache on(1)/off
// Bit 1 - Shared (1) or seperate User/Supervisor address space
// Bit 2 - address monitor mode(1)
reg [2:0]  cache_control = 3'b000;

// Bit 0 - 128MB memory from 0 cacheable(1)/not cachable
// Bit 1 - next 128MB region etc.
reg [31:0] cacheable_area = 32'h0;

// Marks memory regions as read only so writes are ignored by the cache
// Bit 0 - 2MB memory from 0 to 0x01fffff updateable(1)/not updateable
// Bit 1 - next 2MB region etc.
reg [31:0] updateable_area = 32'h0;

// Accesses to a region with a flag set in this register cause the
// cache to flush
// Bit 0 - 2MB memory from 0 to 0x01fffff
// Bit 1 - next 2MB region etc.
reg [31:0] disruptive_area = 32'h0;


reg [7:0]  fault_status  = 'd0;
reg [31:0] fault_address = 'd0;  // the address that caused the fault

wire       copro15_reg1_write;


 //start of generated section from template file
 	reg [31:0] 	acc_in [0:(`ACC_INPUTS - 1)];
 	reg [31:0] 	acc_out[0:(`ACC_OUTPUTS - 1)];
 	reg [31:0] 	acc_opt;
	wire 		acc_poll;
	

	wire [31:0] acc_out_0;
	wire [31:0] acc_out_1;
	// ADDHERE: Additional outputs must be assigned here.

 	
 	reg acc_start;
 	wire acc_rdy;
	integer i;
	
	assign acc_poll = ((!acc_start) && (acc_rdy || acc_poll));
	
	cellacc_IFM accelerator(
 	.i_clk (i_clk),
 	.i_rst (i_rst),
 	.i_start (acc_start),
 	.i_opt (acc_opt),
 	.o_rdy (acc_rdy),
	.acc_in_0 (acc_in[0]),
	.acc_in_1 (acc_in[1]),
	.acc_out_0 (acc_out_0),
	.acc_out_1 (acc_out_1)
	// ADDHERE: Additional I/O-ports must be assigned here
 );
 //end of generated section
 

// ---------------------------
// Outputs
// ---------------------------
assign o_cache_enable   = cache_control[0];
assign o_cache_flush    = copro15_reg1_write;
assign o_cacheable_area = cacheable_area;
              
// ---------------------------
// Capture an access fault address and status
// ---------------------------
always @ ( posedge i_clk )
  if(i_rst)
    begin
       fault_status  <= 'd0;
       fault_address <= 'd0;
    end
  else
    if ( !i_core_stall )
        begin
        if ( i_fault )
            begin
            
            `ifdef A25_COPRO15_DEBUG    
            $display ("Fault status  set to 0x%08x", i_fault_status);
            $display ("Fault address set to 0x%08x", i_fault_address);
            `endif        
            fault_status    <= i_fault_status;
            fault_address   <= i_fault_address;
            end
        end


// ---------------------------
// Register Writes
// ---------------------------
 
 ////start of generated section from template file
 
  always @(posedge acc_rdy)
 begin
	acc_out[0] <= acc_out_0;
	acc_out[1] <= acc_out_1;
	// ADDHERE: If we need more ports, add one here.
 end
 
always @ ( posedge i_clk )
	if(i_rst)
	begin
 		cache_control <= 3'b000;
 		cacheable_area <= 32'h0;
 		updateable_area <= 32'h0;
 		disruptive_area <= 32'h0;
		acc_start <= 'h0;
 	end
 	else
 	begin
		if(acc_start)
			acc_start <= 0;
		if ( !i_core_stall )         
 		begin
 			if ( i_copro_operation == 2'd2 )
 				if (i_copro_num == 'd14) //TAG_ACC
				begin
					if(i_copro_crn == 4'hf)
					begin
						acc_opt <= i_copro_write_data[31:0];
						acc_start <= 1;
					end
					if (i_copro_crn <= (`ACC_INPUTS))
						acc_in[i_copro_crn] <= i_copro_write_data[31:0];
				end
 				else
 					case ( i_copro_crn )
 						 4'd2: cache_control   <= i_copro_write_data[2:0];
 						 4'd3: cacheable_area  <= i_copro_write_data[31:0];
 						 4'd4: updateable_area <= i_copro_write_data[31:0];
 						 4'd5: disruptive_area <= i_copro_write_data[31:0];
 					endcase
 		end
 	end
	
// Flush the cache
assign copro15_reg1_write = !i_core_stall && i_copro_operation == 2'd2 && i_copro_crn == 4'd1 && i_copro_num == 'd15;
 
 // ---------------------------
 // Register Reads   
 // ---------------------------
always @ ( posedge i_clk )        
     if ( !i_core_stall )
 		if (i_copro_num == 'd14)//TAG_ACC
 		begin
			if(i_copro_crn == 4'hf)
				o_copro_read_data <= {31'b0,acc_poll};
			else
				o_copro_read_data <= acc_out[i_copro_crn];
 		end
 		else
         case ( i_copro_crn )
             4'd0:    o_copro_read_data <= 32'h4156_0300;
             4'd2:    o_copro_read_data <= {29'd0, cache_control}; 
             4'd3:    o_copro_read_data <= cacheable_area; 
             4'd4:    o_copro_read_data <= updateable_area; 
             4'd5:    o_copro_read_data <= disruptive_area; 
             4'd6:    o_copro_read_data <= {24'd0, fault_status };
             4'd7:    o_copro_read_data <= fault_address;
             default: o_copro_read_data <= 32'd0;
         endcase
endmodule
