//-----------------------------------------------------------------------------
// Title         : Amber system
// Project       : SHMAC
//-----------------------------------------------------------------------------
// File          : amber_system.v
// Author        : Asbjørn Djupdal  <djupdal@idi.ntnu.no>
// Created       : 06.09.2013
// Last modified : 06.09.2013
//-----------------------------------------------------------------------------
// Description : Wishbone based system containing CPU, timer,
// interrupt controller and tile register
// 
//-----------------------------------------------------------------------------
// Copyright (c) 2013 by ARM/CARD
//------------------------------------------------------------------------------
// Modification history :
// ??.07.2013 : created
//-----------------------------------------------------------------------------

//////////////////////
//////WARNING!!///////
//////////////////////
//
//This file is used as a source file for accelerator interface tile generation!
//It contains tags used for automatic navigation by the generator script.
//make sure any changes made to this file is compatible with the rest of the system
//Files that can cause compatibility issues are listed below 
// /shmac/hardware/tileGenerator/tileGenerator.py
// /shmac/hardware/tileGenerator/templates/Floop_sys_TEMP.v
// /shmac/hardware/tileGenerator/templates/slave_sys_TEMP.v
// /shmac/hardware/tileGenerator/templates/floop_TEMP.v
// /shmac/hardware/tileGenerator/templates/slave_TEMP.v
// /shmac/hardware/tileGenerator/templates/wb_arbiter_TEMP.v
// /shmac/hardware/tileGenerator/templates/wb_arbiter_TEMP.v
// /shmac/hardware/tiles/amber_tile/amber_wrapper.v
// /shmac/hardware/units/amber/hw/vlog/amber25/a25_core.v
//
// Marton Teilgård mteilgard@gmail.com
//////////////////////

`include "common_defs.v"
module amber_system_cellacc
  #(
    parameter tile_x = 4'b0,
    parameter tile_y = 4'b0,
    parameter cpu_id = 8'hff
    )
   (
    input wire          i_clk,
    input wire          i_rst,

    input wire          i_irq,

    input wire          i_system_rdy,

    output wire [31:0]  o_wb_adr,
    output wire [15:0]  o_wb_sel,
    output wire         o_wb_we,
    input wire [127:0]  i_wb_dat,
    output wire [127:0] o_wb_dat,
    output wire         o_wb_cyc,
    output reg          o_wb_stb,
    input wire          i_wb_ack,
    input wire          i_wb_err,
    output wire         o_ld_excl

    );

   ////////////////////////////////////////////////////////////////////////////

   wire                 irq;
   wire                 firq;

   reg [127:0]          wb_dat_r_cpu;
   wire [127:0]         wb_dat_r_tileregs;
   wire [127:0]         wb_dat_r_timer;
   wire [127:0]         wb_dat_r_irq;

   reg                  wb_ack_cpu;
   wire                 wb_ack_tileregs;
   wire                 wb_ack_timer;
   wire                 wb_ack_irq;

   reg                  wb_err_cpu;
   wire                 wb_err_tileregs;
   wire                 wb_err_timer;
   wire                 wb_err_irq;

   wire                 wb_stb_cpu;
   reg                  wb_stb_tileregs;
   reg                  wb_stb_timer;
   reg                  wb_stb_irq;

   wire [2:0]           irq_timers;
   wire [31:1]          int_sources;

   wire [31:0]          tile_base = `TILE_BASE;
   wire [15:0]          tilereg = `TILE_REGS;
   wire [15:0]          timer_mod = `TIMER;
   wire [15:0]          int_ctrl = `INT_CTRL;

   ////////////////////////////////////////////////////////////////////////////
a25_core_cellacc u_amber
     (
      .i_clk          (i_clk),
      .i_rst          (i_rst),
      
      .i_irq          (irq),
      .i_firq         (firq),

      .i_system_rdy   (i_system_rdy),
      
      .o_wb_adr       (o_wb_adr),
      .o_wb_sel       (o_wb_sel),
      .o_wb_we        (o_wb_we),
      .i_wb_dat       (wb_dat_r_cpu),
      .o_wb_dat       (o_wb_dat),
      .o_wb_cyc       (o_wb_cyc),
      .o_wb_stb       (wb_stb_cpu),
      .i_wb_ack       (wb_ack_cpu),
      .i_wb_err       (wb_err_cpu),
      .o_ld_excl      (o_ld_excl)
      );

   tile_regs
     #(
       .WB_DWIDTH (128),
       .WB_SWIDTH (16),
       .tile_x (tile_x),
       .tile_y (tile_y),
       .cpu_id (cpu_id)
       )
   u_tile_regs
     (
      .i_clk (i_clk),
      .i_rst (i_rst),

      .i_wb_adr (o_wb_adr),
      .i_wb_sel (o_wb_sel),
      .i_wb_we (o_wb_we),
      .o_wb_dat (wb_dat_r_tileregs),
      .i_wb_dat (o_wb_dat),
      .i_wb_cyc (o_wb_cyc),
      .i_wb_stb (wb_stb_tileregs),
      .o_wb_ack (wb_ack_tileregs),
      .o_wb_err (wb_err_tileregs)
      );

   timer_module 
     #(
       .WB_DWIDTH (128),
       .WB_SWIDTH (16)
       )
   u_timer
     (
      .i_clk (i_clk),
      .i_rst (i_rst),

      .i_wb_adr (o_wb_adr),
      .i_wb_sel (o_wb_sel),
      .i_wb_we (o_wb_we),
      .o_wb_dat (wb_dat_r_timer),
      .i_wb_dat (o_wb_dat),
      .i_wb_cyc (o_wb_cyc),
      .i_wb_stb (wb_stb_timer),
      .o_wb_ack (wb_ack_timer),
      .o_wb_err (wb_err_timer),
      .o_timer_int (irq_timers)
      );

   interrupt_controller  
     #(
       .WB_DWIDTH (128),
       .WB_SWIDTH (16)
       )
   u_irq_ctrl
     (
      .i_clk (i_clk),
      .i_rst (i_rst),

      .i_wb_adr (o_wb_adr),
      .i_wb_sel (o_wb_sel),
      .i_wb_we (o_wb_we),
      .o_wb_dat (wb_dat_r_irq),
      .i_wb_dat (o_wb_dat),
      .i_wb_cyc (o_wb_cyc),
      .i_wb_stb (wb_stb_irq),
      .o_wb_ack (wb_ack_irq),
      .o_wb_err (wb_err_irq),
      
      .o_irq (irq),
      .o_firq (firq),
      
      .i_int_sources (int_sources)
      );
	
	 //ACCTAGINST 
	
   ////////////////////////////////////////////////////////////////////////////

   assign int_sources = {27'b0, irq_timers, i_irq};
	 //ACCTAGINSTDONE 
   always @* begin
      // default is router
      wb_dat_r_cpu = i_wb_dat;
      wb_ack_cpu = i_wb_ack;
      wb_err_cpu = i_wb_err;
      o_wb_stb = wb_stb_cpu;
      wb_stb_tileregs = 0;
      wb_stb_timer = 0;
      wb_stb_irq = 0;
	  //ACCTAGSTBRST

      // override default for local wishbone addresses
      if(o_wb_adr[31:16] == tile_base[31:16]) begin
         case(o_wb_adr[15:12])
           tilereg[15:12]: begin
              wb_dat_r_cpu = wb_dat_r_tileregs;
              wb_ack_cpu = wb_ack_tileregs;
              wb_err_cpu = wb_err_tileregs;
              o_wb_stb = 0;
              wb_stb_tileregs = wb_stb_cpu;
           end

           timer_mod[15:12]: begin
              wb_dat_r_cpu = wb_dat_r_timer;
              wb_ack_cpu = wb_ack_timer;
              wb_err_cpu = wb_err_timer;
              o_wb_stb = 0;
              wb_stb_timer = wb_stb_cpu;
           end

           int_ctrl[15:12]: begin
              wb_dat_r_cpu = wb_dat_r_irq;
              wb_ack_cpu = wb_ack_irq;
              wb_err_cpu = wb_err_irq;
              o_wb_stb = 0;
              wb_stb_irq = wb_stb_cpu;
           end
		   //ACCTAGSTB 
         endcase
      end
        
   end

endmodule
