library ieee;
use ieee.std_logic_1164.all;

use work.types.all;
use work.components.all;

--//////////////////////
--//////WARNING!!///////
--//////////////////////
--//
--//This file is used as a source file for accelerator interface tile generation!
--//It contains tags used for automatic navigation by the generator script.
--//make sure any changes made to this file is compatible with the rest of the system
--//Files that can cause compatibility issues are listed below 
--// /shmac/hardware/tileGenerator/tileGenerator.py
--// /shmac/hardware/tiles/amber_tile/amber_wrapper.v
--//
--// Marton Teilgård mteilgard@gmail.com
--//////////////////////

-------------------------------------------------------------------------------
-- AMBER Tile
-------------------------------------------------------------------------------
entity amber_tile_cellacc is
  generic
  (
    coord : tile_coord := (i => 0, j => 0);
    apb_i : integer;
    apb_j : integer;
    zbt_i : integer;
    zbt_j : integer;
    bram0_i : integer;
    bram0_j : integer;
    bram1_i : integer;
    bram1_j : integer;
    bram2_i : integer;
    bram2_j : integer;
    bram3_i : integer;
    bram3_j : integer;
    bram4_i : integer;
    bram4_j : integer;
    bram5_i : integer;
    bram5_j : integer;
    bram6_i : integer;
    bram6_j : integer;
    bram7_i : integer;
    bram7_j : integer;
    cpu_id : natural
  );

  port
  (
    clk   : in  std_logic;

    reset_cpu : in  std_logic;
    reset_ic : in  std_logic;
    
    irq : in  std_logic;

    system_rdy : in std_logic;

    rx_req_vector    : in  direction_boolean_vector;
    rx_packet_vector : in  packet_vector;
    rx_ack_vector    : out direction_boolean_vector;
    
    tx_req_vector    : out direction_boolean_vector;
    tx_packet_vector : out packet_vector;
    tx_ack_vector    : in  direction_boolean_vector
  );
end amber_tile_cellacc;

architecture rtl of amber_tile_cellacc is

  -- Amber
  signal amber_rx_req         : std_logic;
  signal amber_rx_packet      : std_logic_vector(sizeof_local_pkt-1 downto 0);
  signal amber_rx_ack         : std_logic;
  
  signal amber_tx_req         : std_logic;
  signal amber_tx_packet      : std_logic_vector(sizeof_local_pkt-1 downto 0);
  signal amber_tx_ack         : std_logic;

  -- Router
  signal rx_req_vector_i    : std_logic_vector(dir_cnt-1 downto 0);
  signal rx_packet_vector_i : std_logic_vector(sizeof_pkt_vec-1 downto 0);
  signal rx_ack_vector_i    : std_logic_vector(dir_cnt-1 downto 0);
  
  signal tx_req_vector_i    : std_logic_vector(dir_cnt-1 downto 0);
  signal tx_packet_vector_i : std_logic_vector(sizeof_pkt_vec-1 downto 0);
  signal tx_ack_vector_i    : std_logic_vector(dir_cnt-1 downto 0);

  -- Netiface
  signal ni_rx_req         : std_logic;
  signal ni_rx_packet      : std_logic_vector(sizeof_pkt-1 downto 0);
  signal ni_rx_ack         : std_logic;
  
  signal ni_tx_req         : std_logic;
  signal ni_tx_packet      : std_logic_vector(sizeof_pkt-1 downto 0);
  signal ni_tx_ack         : std_logic;

  --attribute keep : string;
  --attribute keep of amber_rx_req     : signal is "true";
  --attribute keep of amber_rx_packet  : signal is "true";
  --attribute keep of amber_rx_ack     : signal is "true";
  --attribute keep of amber_tx_req     : signal is "true";
  --attribute keep of amber_tx_packet  : signal is "true";
  --attribute keep of amber_tx_ack     : signal is "true";
  
begin
amber_u: amber_wrapper_cellacc
    generic map (
      tile_x => coord.j,
      tile_y => coord.i,
      cpu_id => cpu_id
      )
    port map (
      clk       => clk,
      rst       => reset_cpu,
      irq       => irq,
      system_rdy => system_rdy,
      req_in    => amber_rx_req,
      data_in   => amber_rx_packet,
      rdy_in    => amber_rx_ack,
      req_out   => amber_tx_req,
      data_out  => amber_tx_packet,
      rdy_out   => amber_tx_ack
    );

  netiface_u: netiface
    generic map (
      tile_x => coord.j,
      tile_y => coord.i,
      apb_i => apb_i,
      apb_j => apb_j,
      zbt_i => zbt_i,
      zbt_j => zbt_j,
      bram0_i => bram0_i,
      bram0_j => bram0_j,
      bram1_i => bram1_i,
      bram1_j => bram1_j,
      bram2_i => bram2_i,
      bram2_j => bram2_j,
      bram3_i => bram3_i,
      bram3_j => bram3_j,
      bram4_i => bram4_i,
      bram4_j => bram4_j,
      bram5_i => bram5_i,
      bram5_j => bram5_j,
      bram6_i => bram6_i,
      bram6_j => bram6_j,
      bram7_i => bram7_i,
      bram7_j => bram7_j
      )
    port map (
      clk       => clk,
      rst       => reset_ic,
      req_in    => amber_tx_req,
      data_in   => amber_tx_packet,
      rdy_in    => amber_tx_ack,
      req_out   => amber_rx_req,
      data_out  => amber_rx_packet,
      rdy_out   => amber_rx_ack,
      req_in_r    => ni_rx_req,
      data_in_r   => ni_rx_packet,
      rdy_in_r    => ni_rx_ack,
      req_out_r   => ni_tx_req,
      data_out_r  => ni_tx_packet,
      rdy_out_r   => ni_tx_ack
    );

  router_u: router
    generic map (
      tile_x => coord.j,
      tile_y => coord.i
      )
    port map (
      clk       => clk,
      rst       => reset_ic,
      req_in    => rx_req_vector_i,
      data_in   => rx_packet_vector_i,
      rdy_in    => rx_ack_vector_i,
      req_out   => tx_req_vector_i,
      data_out  => tx_packet_vector_i,
      rdy_out   => tx_ack_vector_i
    );


  -- Local
  rx_req_vector_i(LOCAL_DIR)            <= ni_rx_req;
  rx_packet_vector_i(LOCAL_PACKET_END
    downto LOCAL_PACKET_BEGIN)          <= ni_rx_packet;
  ni_rx_ack                             <= rx_ack_vector_i(LOCAL_DIR);

  ni_tx_req                             <= tx_req_vector_i(LOCAL_DIR);
  ni_tx_packet                          <= tx_packet_vector_i(LOCAL_PACKET_END
                                             downto LOCAL_PACKET_BEGIN);
  tx_ack_vector_i(LOCAL_DIR)            <= ni_tx_ack;

  -- East
  rx_req_vector_i(EAST_DIR)             <= boolean_to_std_logic(rx_req_vector(east));
  rx_packet_vector_i(EAST_PACKET_END
    downto EAST_PACKET_BEGIN)           <= rx_packet_vector(east);
  rx_ack_vector(east)                   <= std_logic_to_boolean(rx_ack_vector_i(EAST_DIR));

  tx_req_vector(east)                   <= std_logic_to_boolean(tx_req_vector_i(EAST_DIR));
  tx_packet_vector(east)                <= tx_packet_vector_i(EAST_PACKET_END
                                             downto EAST_PACKET_BEGIN);
  tx_ack_vector_i(EAST_DIR)             <= boolean_to_std_logic(tx_ack_vector(east));

  -- North
  rx_req_vector_i(NORTH_DIR)             <= boolean_to_std_logic(rx_req_vector(north));
  rx_packet_vector_i(NORTH_PACKET_END
    downto NORTH_PACKET_BEGIN)           <= rx_packet_vector(north);
  rx_ack_vector(north)                   <= std_logic_to_boolean(rx_ack_vector_i(NORTH_DIR));

  tx_req_vector(north)                   <= std_logic_to_boolean(tx_req_vector_i(NORTH_DIR));
  tx_packet_vector(north)                <= tx_packet_vector_i(NORTH_PACKET_END
                                             downto NORTH_PACKET_BEGIN);
  tx_ack_vector_i(NORTH_DIR)             <= boolean_to_std_logic(tx_ack_vector(north));

  -- West
  rx_req_vector_i(WEST_DIR)             <= boolean_to_std_logic(rx_req_vector(west));
  rx_packet_vector_i(WEST_PACKET_END
    downto WEST_PACKET_BEGIN)           <= rx_packet_vector(west);
  rx_ack_vector(west)                   <= std_logic_to_boolean(rx_ack_vector_i(WEST_DIR));

  tx_req_vector(west)                   <= std_logic_to_boolean(tx_req_vector_i(WEST_DIR));
  tx_packet_vector(west)                <= tx_packet_vector_i(WEST_PACKET_END
                                             downto WEST_PACKET_BEGIN);
  tx_ack_vector_i(WEST_DIR)             <= boolean_to_std_logic(tx_ack_vector(west));

  -- South
  rx_req_vector_i(SOUTH_DIR)             <= boolean_to_std_logic(rx_req_vector(south));
  rx_packet_vector_i(SOUTH_PACKET_END
    downto SOUTH_PACKET_BEGIN)           <= rx_packet_vector(south);
  rx_ack_vector(south)                   <= std_logic_to_boolean(rx_ack_vector_i(SOUTH_DIR));

  tx_req_vector(south)                   <= std_logic_to_boolean(tx_req_vector_i(SOUTH_DIR));
  tx_packet_vector(south)                <= tx_packet_vector_i(SOUTH_PACKET_END
                                             downto SOUTH_PACKET_BEGIN);
  tx_ack_vector_i(SOUTH_DIR)             <= boolean_to_std_logic(tx_ack_vector(south));

end rtl;
