//-----------------------------------------------------------------------------
// Title         : Amber wrapper
// Project       : SHMAC
//-----------------------------------------------------------------------------
// File          : amber_wrapper.v
// Author        : Asbjørn Djupdal  <djupdal@idi.ntnu.no>
// Created       : ??.07.2013
// Last modified : 23.08.2013
//-----------------------------------------------------------------------------
// Description :
// 
//-----------------------------------------------------------------------------
// Copyright (c) 2013 by ARM/CARD
//------------------------------------------------------------------------------
// Modification history :
// ??.07.2013 : created
//-----------------------------------------------------------------------------

//////////////////////
//////WARNING!!///////
//////////////////////
//
//This file is used as a source file for accelerator interface tile generation!
//It contains tags used for automatic navigation by the generator script.
//make sure any changes made to this file is compatible with the rest of the system
//Files that can cause compatibility issues are listed below 
// /shmac/hardware/tileGenerator/tileGenerator.py
// /shmac/hardware/tileGenerator/templates/Floop_sys_TEMP.v
// /shmac/hardware/tileGenerator/templates/slave_sys_TEMP.v
// /shmac/hardware/tiles/amber_tile/amber_system.v
// /shmac/hardware/tiles/amber_tile/amber_tile.vhd
//
// Marton Teilgård mteilgard@gmail.com
//////////////////////

`include "common_defs.v"
module amber_wrapper_cellacc
  #(
    parameter tile_x = 4'b0,
    parameter tile_y = 4'b0,
    parameter cpu_id = 8'hff
    )
  (
   input wire                      clk,
   input wire                      rst,

   input wire                      irq,

   input wire                      system_rdy,

   // network interface
   input wire                      req_in,
   output wire                     rdy_in,
   input wire [`PACKET_WIDTH-1:0]  data_in,

   output reg                      req_out,
   input wire                      rdy_out,
   output wire [`PACKET_WIDTH-1:0] data_out
   );

   ////////////////////////////////////////////////////////////////////////////
    // FSM

   `define st_idle 2'h0
   `define st_read 2'h1
   `define st_ack 2'h2
   
   reg [1:0]                       state;
   reg [1:0]                       next_state;

   ////////////////////////////////////////////////////////////////////////////
   // wishbone signals

   wire [31:0]                     wb_adr;
   wire [15:0]                     wb_sel;
   wire                            wb_we;
   reg [127:0]                     wb_dat_r;
   wire [127:0]                    wb_dat_w;
   wire                            wb_cyc;
   wire                            wb_stb;
   reg                             wb_ack;
   wire                            wb_err;
   wire                            ld_excl;

   ////////////////////////////////////////////////////////////////////////////
   
   wire ld_error;
	amber_system_cellacc
     #(
       .tile_x (tile_x),
       .tile_y (tile_y),
       .cpu_id (cpu_id)
       )
   u_amber_system
     (
      .i_clk          (clk),
      .i_rst          (rst),
      
      .i_irq          (irq),

      .i_system_rdy   (system_rdy),
      
      .o_wb_adr       (wb_adr),
      .o_wb_sel       (wb_sel),
      .o_wb_we        (wb_we),
      .i_wb_dat       (wb_dat_r),
      .o_wb_dat       (wb_dat_w),
      .o_wb_cyc       (wb_cyc),
      .o_wb_stb       (wb_stb),
      .i_wb_ack       (wb_ack),
      .i_wb_err       (wb_err),
      .o_ld_excl      (ld_excl)
      );

   assign data_out[`ADDR_END:`ADDR_BEGIN]       = {wb_adr[31:4],4'b0};
   assign data_out[`REPLY_IDX]                  = 1'b0;
   assign data_out[`WR_REQ_IDX]                 = wb_we;
   assign data_out[`WR_MASK_END:`WR_MASK_BEGIN] = wb_sel;
   assign data_out[`EXCL_IDX]                   = ld_excl;
   assign data_out[`ERROR_IDX]                  = 1'b0;
   assign data_out[`DATA_END:`DATA_BEGIN]       = wb_dat_w;

   assign wb_err = 1'b0;

   assign rdy_in = 1'b1;
   
   assign ld_error = data_in[`ERROR_IDX];

   always @(posedge clk or posedge rst) begin
      if(rst)
        state <= `st_idle;
      else 
        state <= next_state;
   end

   always @* begin
      next_state <= state;

      req_out <= 0;
      wb_ack <= 0;

      wb_dat_r <= data_in[`DATA_END:`DATA_BEGIN];

      case(state)

        `st_idle: begin
           if(wb_stb) begin
              req_out <= 1;

              if(rdy_out)
                if(wb_we)
                  next_state <= `st_ack;
                else
                  next_state <= `st_read;
           end
        end

        `st_read: begin
           if (req_in) begin
              next_state <= `st_idle;

              // This is for a non-exclusive or a successful exclusive load.
              // In case of an unsuccessful exclusive load -
              // go to st_idle to resend the request (wb_ack remains 0).
              if (~ld_excl | (ld_excl & ~ld_error))
                wb_ack <= 1;
           end
        end

        `st_ack: begin
           wb_ack <= 1;
           next_state <= `st_idle;
        end

      endcase
   end

endmodule
