#include <sstream>
#include <cassert>
#include <iostream>
#include "cell.h"
#include "rules.h"
using namespace std;

Cell::Cell(unsigned int neighbourness, CellInterface *comm, unsigned int x, unsigned int y) : _state(0), _nextState(0),
	_numNeighbours(0),
	_comm(comm),
	_x(x),
	_y(y) {
	_neighbours = new const Cell*[neighbourness];
	for (unsigned int i = 0; i < neighbourness; i++) {
		_neighbours[i] = nullptr;
	}
	_neighbourness = neighbourness;
	_state = -1;
	_nextState = -1;
}

Cell::~Cell() {
	delete[] _neighbours;
}

void Cell::setNumNeighbours(unsigned int num) {
	assert(num >= 0);
	assert(num <= _neighbourness);
	_numNeighbours = num;
}

void Cell::setNeighbour(int relation, const Cell * const neighbour) {
	_neighbours[relation] = neighbour;
}

void Cell::clearAllNeighbours() {
	for (int i = 0; i < _numNeighbours; i++) {
		_neighbours[i] = nullptr;
	}
}

int Cell::getState() const {
	return _state;
}

void Cell::setState(int state) {
	_state = state;
}

std::string Cell::getStateAsString() const {
	std::stringstream ss;
	if (_state == -1) {
		return "X";
	} else {
		ss << getState();
	}
	return ss.str();
}

std::string Cell::getStateString() const {
	char *stateArr = new char[_numNeighbours + 1];
	stateArr[_numNeighbours] = '\0';
	for (int i = 0; i < _numNeighbours; i++) {
		if (_neighbours[i] != nullptr) {
			string state = _neighbours[i]->getStateAsString();
			assert(state.size() == 1);
			stateArr[i] = state.at(0);
		} else {
			stateArr[i] = '0';
		}
	}
	string result(stateArr);
	delete[] stateArr;
	return result;
}

void Cell::update() {
	assert(_comm);
	_nextState = _comm->getNextState(getStateString(), _x, _y);
}

void Cell::tick() {
	_state = _nextState;
}
