#ifndef CELL_H
#define CELL_H

#include <string>

class CellInterface;

class Cell {
private:
	unsigned int _x, _y;
	int _state;
	int _nextState;
	const Cell **_neighbours; // Order: NW, SW, N, W, C, E, S, NE, SE;
	unsigned int _numNeighbours;
	const CellInterface *_comm;
	unsigned int _neighbourness;
public:
	/**
	 * Constructor
	 * @param neighbourness the amount of neighbours taken into account.
	 * @param comm a pointer to the communicator that this cell will use.
	 * @param x the x coordinate (used for non-uniform rule handling).
	 * @param y the y coordinate (used for non-uniform rule handling).
	 */
	Cell(unsigned int neighbourness, CellInterface *comm, unsigned int x, unsigned int y);
	~Cell();
	void setNumNeighbours(unsigned int num);
	void setNeighbour(int relation, const Cell * const neighbour);
	void clearAllNeighbours();
	const Cell *getNeighbour(int relation) const { return _neighbours[relation]; }
	int getState() const;
	void setState(int state);
	std::string getStateAsString() const;
	std::string getStateString() const;
	void update();
	void tick();
};

#endif
