#include <vector>
#include <string>
#include <list>

#ifndef cellconv_opcodes_h
#define cellconv_opcodes_h

enum OPCODES {
	// Actual coproc opcodes:
	OP_HALT = 0,      // 000
	OP_RUN = 1,       // 001
	OP_SETRULE = 4,   // 100
	OP_SETEDGE = 5,   // 101
	OP_LOADCOL = 6,   // 110
	OP_RST = 7,	      // 111
	// Macros:
	OP_ASSERT,
	OP_LOADFILE,
	OP_HEIGHT,		  // Settings (do something on both simulator and asm)
	OP_WIDTH,
	OP_NEIGHBOURS,
	OP_VERIFY,
	OP_VERIFY_ALL,    // Side-effect: SetEdge(1,1,0).
	// Assembly only:
	OP_MAKEFUNC,
	// Simulator only:
	OP_PRINT,
	// Simulator only settings
	CONF_LST,
	CONF_PRINT,
};

class Program;

class Operation {
	std::vector<std::string> _tokens;
	// Queue of results
	std::list<std::string> _results;
public:
	Operation(const std::string &instruction);
	void assertNumArgs(int num) const;
	std::string getArg(int num) const;
	int getIntArg(int num) const;
	bool isMacro() const;
	OPCODES getOpcode() const;
	void pushResult(const std::string &str);
	std::string popResult();
	// Currently this actually eats the stack of this op.
	std::string getAssemblyForOperation(Program *prog, bool useRunBlock = true);
};

class Program {
	std::vector<Operation> _operations;
	int _height;
	int _width;
	int _numNeighbours;
	void handleMacro(const Operation &op);
public:
	Program(const std::string& str);
	std::string convertToArmAssembly(bool generateRunBlock);
	void runInSimulator(); // sim.cpp
	
	int getHeight() const { return _height; }
	int getWidth() const { return _width; }
	int getNumNeighbours() const { return _numNeighbours; }
};

void runProgram(std::string filename);

#endif
