#ifndef RULES_H
#define RULES_H

#include <string>
#include <map>

/** The cell's interface to the outer world, allows us to change the ruleset without telling the Cells */
class CellInterface {
public:
	virtual ~CellInterface() {}
	virtual int getNextState(std::string currentState, unsigned int x, unsigned int y) const = 0;
};

enum RuleMode {
	MODE_NORMAL,
	MODE_LOAD,
	MODE_RESET
};

// In practice, these two classes are merged in VHDL.
class Rules {
	std::map<std::string, int> _ruleTable;

public:
	Rules();
	// Normal operation:
	void addRule(std::string key, int nextState);
	void addKeyValRule(std::string key, std::string value);
	void extendToNeighbours(unsigned int neighbourness);
	int getNextState(std::string key, unsigned int x, unsigned int y) const;
	int getSize() const;
	enum PrintModes {
		PRINT_VHDL,
		PRINT_PSEUDO_ASSEMBLY
	};
	void printAs(PrintModes printMode) const;
	void printAsVHDLTestBench() const;
	void printAsPseudoAssembly() const;
};

class CellCommunicator : public CellInterface {
	RuleMode _mode;
	
	const int *_loadTable;
	int _loadTableSize;
	int _resetValue;
	
	const Rules *_ruleTable;
public:
	CellCommunicator() {
		_ruleTable = nullptr;
		_mode = MODE_NORMAL;
		_resetValue = 0;
		_loadTable = nullptr;
		_loadTableSize = 0;
	}
	virtual ~CellCommunicator() {}
	void setRuleSet(const Rules *rules) {
		_ruleTable = rules;
	}

	const Rules *getRuleSet() { return _ruleTable; }
	void setResetValue(bool value) { _resetValue = value; }
	int getNextState(std::string currentState, unsigned int x, unsigned int y) const override;
	void setMode(RuleMode mode) { _mode = mode; }
	RuleMode getMode() const { return _mode; }
};

#endif
