#include <cassert>
#include <vector>
#include "rulesets.h"
#include "rules.h"
#include "gameoflife.h"
#include "utils.h"

using namespace std;

Rules *generateWolframRule(int number) {
	assert(number >= 0 && number <= 255);
	Rules *rule = new Rules();
	rule->addRule("111", (number >> 7) & 1);	
	rule->addRule("110", (number >> 6) & 1);	
	rule->addRule("101", (number >> 5) & 1);	
	rule->addRule("100", (number >> 4) & 1);	
	rule->addRule("011", (number >> 3) & 1);	
	rule->addRule("010", (number >> 2) & 1);	
	rule->addRule("001", (number >> 1) & 1);	
	rule->addRule("000", number & 1);
	return rule;
}

Rules *generateRuleSet(ruleType type) {
	switch (type) {
	case RULE_GAMEOFLIFE:
		return generateGameOfLifeRules();	
	case RULE_30:
		return generateWolframRule(30);
	case RULE_90:
		return generateWolframRule(90);
	}
}

// Arbitrarily numbered
Rules *generateTestRuleSet(int number, int neighbourness) {
	std::vector<std::string> temp;
	generateAllPossibleStrings(temp, "", neighbourness - 1);
	Rules *rule = new Rules();
	int NW, SW, N, W, C, E, S, NE, SE;
	C = (neighbourness / 2);
	NW = C - 4;
	SW = C - 3;
	N =  C - 2;
	W =  C - 1;
	E =  C + 1;
	S =  C + 2;
	NE = C + 3;
	SE = C + 4;

	// North to south
	if (number < 9) {
		int direction;
		switch (number) {
			case 8: direction = NW; break;
			case 7: direction = SW; break;
			case 6: direction = N; break;
			case 5: direction = W; break;
			case 4: direction = C; break;
			case 3: direction = E; break;
			case 2: direction = S; break;
			case 1: direction = NE; break;
			case 0: direction = SE; break;
			default: direction = C; break;

		}
		int nextState = 0;
		for (auto it = temp.begin(); it != temp.end(); ++it) {
			string s = *it;
			if (s[direction] == '1') {
				nextState = 1;
			} else {
				nextState = 0;
			}
			rule->addRule(s, nextState);
		}
	}
	return rule;
}
