#ifndef SYSTEM_H
#define SYSTEM_H

#include <vector>
#include <string>

class Cell;
class Rules;
class CellCommunicator;

enum printMode {
	PRINTMODE_LST,
	PRINTMODE_TERM
};

class System {
	Cell ***_cells;
	CellCommunicator *_communicator;
	int *_loadTable;
	void setupNeighbourhood(bool);
	void createCells();
	void destroyCells();
	unsigned int _width, _height, _neighbourness;
	bool _wrapNS, _wrapEW;
	bool _unique;
	printMode _printMode;
	bool _printContinuously;
	std::vector<std::vector<std::string> > _history;
	void logHistory();
public:

	System(unsigned int width, unsigned int height, unsigned int neighbourness, bool nonUniform = false);
	~System();

	/**
	 * @brief Enables/Disables printing only unique generations
	 *
	 * Unique generations in this context, are generations that differ
	 * from the directly preceding generation.
	 */
	void setPrintUniqueOnly(bool value) { _unique = value; }
	void setPrintContinuously(bool value) { _printContinuously = value; }
	void setPrintMode(printMode mode) { _printMode = mode; }
	void setRuleSet(const Rules *ruleset);
	void resetSystemTo(int value);
	void loadColumnIntoSystem(const int *column, size_t height);
	void loadArrayIntoSystem(const int *array, unsigned int width, unsigned int height);
	void loadFileIntoSystem(const std::string &filename);
	unsigned int getWidth() const { return _width; }
	unsigned int getHeight() const { return _height; }
	unsigned int getNeighbourness() const { return _neighbourness; }
	std::string getOutputColumn();
	void shiftSystemEastwards();
	// Wrap-around settings:
	void setWrapEW(bool value);
	void setWrapNS(bool value);
	bool getWrapEW() const { return _wrapEW; }
	bool getWrapNS() const { return _wrapNS; }
	void updateSystem();
	void printSystem() const;
	void printHistory() const;
	void clearHistory();
	void outputStateAsVHDLTB(bool includeRules);
	void outputStateAsPseudoAssembly(bool includeRules);
};

#endif
