#include <iostream>
#include <vector>
#include <string>
#include "utils.h"

using namespace std;

int *create1DArray(int width, int height) {
	int *array = new int[width * height];
	for (int i = 0; i < width * height; i++) {
		array[i] = 0;
	}
	return array;
}

int **create2DArray(int width, int height) {
	int **array = new int*[width];
	for (int i = 0; i < width; i++) {
		array[i] = new int[height];
		for (int j = 0; j < height; j++) {
			array[i][j] = 0;
		}
	}
	return array;
}

unsigned int get1DIndex(unsigned int x, unsigned int y, unsigned int width) {
	return (y * width) + x;
}

void print1DArray(int *array, unsigned int width, unsigned int height) {
	for (unsigned int y = 0; y < height; y++) {
		for (unsigned int x = 0; x < width; x++) {
			cout << array[get1DIndex(x, y, width)];
		}
		cout << endl;
	}
}

void generateAllPossibleStrings(std::vector<std::string> &result, std::string prefix, int depth) {
	if (depth != 0) {
		generateAllPossibleStrings(result, prefix + "1", depth - 1);
		generateAllPossibleStrings(result, prefix + "0", depth - 1);
	} else {
		result.push_back(prefix + "1");
		result.push_back(prefix + "0");
	}
}

string trimWhiteSpace(const string &str) {
	string result = "";
	for (int i = 0; i < str.length(); i++) {
		char sym = str.at(i);
		if (sym != ' ' && sym != '\t' && sym != '\r' && sym && sym != '\n') {
			result += sym;
		}
	}
	return result;
}

// This does allow unterminated comments, but we can live with that.
string trimComments(const string &str) {
	string result = "";
	size_t curPos = 0;
	size_t startPos = str.find("/*");
	
	while (curPos != string::npos) {
		result += str.substr(curPos, startPos - curPos);
		curPos = str.find("*/", startPos);
		if (curPos != string::npos) {
			curPos += 2;
		}
		startPos = str.find("/*", curPos);
	}
	
	return result;
}

vector<string> tokenizeString(const string& str, const string& splitSyms) {
	int startToken = 0;
	int tokenLength = 0;
	vector<string> tokens;
	for (int i = 0; i < str.length(); i++) {
		if (splitSyms.find(str.at(i)) != string::npos) {
			tokens.push_back(str.substr(startToken, tokenLength));
			startToken = i + 1;
			tokenLength = 0;
		} else {
			tokenLength++;
		}
	}
	return tokens;
}

string int2bin(int value, int numBits) {
	string result;
	for (int i = numBits - 1; i >= 0; i--) {
		if (value & (1 << i)) {
			result += "1";
		} else {
			result += "0";
		}
	}
	return result;
}

int bin2dec(const string &str) {
	int value = 0;
	
	for (int i = 0; i < str.size(); i++) {
		size_t exp = str.size() - 1 - i;
		if (str.at(i) == '1') {
			value += 1 << exp;
		}
	}
	return value;
}
