#include <iostream>
#include <fstream>
#include <list>
#include <string>
#include <sstream>
#include <cmath>
#include <vector>
#include <algorithm>
#include <iomanip>

using namespace std;

typedef unsigned int uint32_t;

void statistics(vector<uint32_t> &numbers, int numBits) {
	if (numbers.size() == 0) {
		return;
	}
	double expected = pow(2,numBits) - 1;
	expected /= 2;
	double average = 0.0f;
	for (vector<uint32_t>::iterator it = numbers.begin(); it != numbers.end(); ++it) {
		average += *it;
	}
	average /= numbers.size();
	sort(numbers.begin(), numbers.end());
	cout << "Statistics:" << endl;
	cout << "Numbers found: " << numbers.size() << endl;
	cout << "Average: " << fixed << setprecision(4) << average <<  " Expected: " << expected << " Diff: " << fabs(average-expected) << " Average/Expected: " << average/expected << endl;
	cout << "Median: " << numbers[numbers.size()/2] << endl;
}

int main(int argc, char** argv) {
	printf("Usage: convertBytes [filename] [numBits] -s[skipNum]\nnumBits - the amount of bits to group into numbers\nskipNum - the amount of sequences to skip between numbers (usefull when doing 2D)\n");
	if (argc == 1) {
		cout << "No filename specified" << endl;
		return -1;
	}
	int skipNum = 0;
	if (argc >= 4) {
		if (*argv[3] == '-') {
			string arg(argv[3]);
			if (arg.size() > 2 && arg.at(1) == 's') {
				string num = arg.substr(2, string::npos);
				skipNum = atoi(num.c_str());
			}
			cout << "Skipping sequences of " << skipNum << " elements." << endl;
		}
	}
	int numBits = 8;
	if (argc >= 3) {
		numBits = atoi(argv[2]);
		cout << "Grouping by " << numBits << " bits " << endl;
	}
	fstream file(argv[1]);
	if (!file.good()) {
		cout << "Unable to open file " << argv[1] << endl;
		return -1;
	}

	list<uint32_t> numbers;
	int numNumbers = 0;
	while (!file.eof()) {
		std::stringstream ss;
		string line;
		getline(file, line);
		// Seek backwards:
		bool foundNum = false;
		for (int i = line.size() - 1; i >= 0; i--) {
			if (isdigit(line.at(i))) {
				ss << line.at(i);
				foundNum = true;
				break;
			}
		}
		if (!foundNum) {
			continue;
		}
		uint32_t number = 0;
		ss >> number;
		numbers.push_back(number);
		numNumbers++;
	}
	cout << "Initially read " << numNumbers << " numbers. " << endl;

	vector<uint32_t> convertedNums;
	int idx = 0;
	while (numNumbers >= numBits) {
		uint32_t num = 0;
		for (int i = numBits - 1; i >= 0; i--) {
			if (skipNum && idx % skipNum == 0) {
				cout << numbers.front() << " ";
			}
			num += numbers.front() * pow(2,i);
			numbers.pop_front();
			numNumbers--;
		}
		if (skipNum == 0 || idx % skipNum == 0) {
			cout << " : " << num << endl;
			convertedNums.push_back(num);
		}
		idx++;
	}

	cout << "Remainder: ";
	for (int i = 0; i < numNumbers; i++) {
		if (i % skipNum == 0) {
			cout << numbers.front() << " ";
		} else {
			cout << "Skipped: " << numbers.front() << " ";
		}
		numbers.pop_front();
	}
	cout << endl;
	statistics(convertedNums, numBits);

	return 0;
}
