#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>

using namespace std;

void copy4Bytes(unsigned char *dest, unsigned char *src) {
	dest[0] = src[0];
	dest[1] = src[1];
	dest[2] = src[2];
	dest[3] = src[3];
}

unsigned int parseLine(string line, unsigned char *target) {
	unsigned char XColour[4] = {0,0,255,255};
	unsigned char ZColour[4] = {0,255,0,255};
	unsigned char UColour[4] = {255,0,0,255};
	unsigned char colour1[4] = {0,0,0,0};
	unsigned char colour0[4] = {255,255,255,255};

	unsigned int bytes = 0;
	for (unsigned int i = 0; i < line.size(); i++) {
		if (line[i] == 'X') {
			copy4Bytes(target, XColour);
			target += 4;
			bytes++;
		} else if (line[i] == 'Z') {
			copy4Bytes(target, ZColour);
			target += 4;
			bytes++;
		} else if (line[i] == 'U') {
			copy4Bytes(target, UColour);
			target += 4;
			bytes++;
		} else {
			if (line[i] == '1') {
				copy4Bytes(target, colour1);
				target += 4;
				bytes++;
			} else if (line[i] == '0') {
				copy4Bytes(target, colour0);
				target += 4;
				bytes++;
			}
		}
	}
	return bytes;
}

void writeBMP(const char* filename, unsigned int width, unsigned int height, const unsigned char *data) {
	ofstream bmp(filename, ios::binary);

	unsigned int dataSize = width * height * 4;

	const unsigned int BMPHeaderSize = 14;
	const unsigned int DIBHeaderSize = 40;
	// Write header
	unsigned short reserved = 0;
	char b = 'B';
	char m = 'M';
	unsigned int fileSize = BMPHeaderSize + DIBHeaderSize + dataSize*4;
	unsigned int startAddr = BMPHeaderSize + DIBHeaderSize + 2;

	bmp.write(&b, 1);
	bmp.write(&m, 1);
	bmp.write((const char*)&fileSize, 4);
	bmp.write((const char*)&reserved, 2);
	bmp.write((const char*)&reserved, 2);
	bmp.write((const char*)&startAddr, 4);

	// DIB-header
	unsigned int dibHeaderSize = 40;
	unsigned int dibImgWidth = width;
	unsigned int dibImgHeight = height;
	unsigned short dibNumColourPlanes = 1;
	unsigned short dibBPP = 32;
	unsigned int dibCompressionType = 0; // NO compression
	unsigned int dibImgSize = 0;
	unsigned int dibHorRez = 0;
	unsigned int dibVerRez = 0;
	unsigned int dibNumColoursInPalette = 0;
	unsigned int dibNumImportantColours = 0;

	bmp.write((const char*)&dibHeaderSize, 4);
	bmp.write((const char*)&dibImgWidth, 4);
	bmp.write((const char*)&dibImgHeight, 4);
	bmp.write((const char*)&dibNumColourPlanes, 2);
	bmp.write((const char*)&dibBPP, 2);
	bmp.write((const char*)&dibCompressionType, 4);
	bmp.write((const char*)&dibImgSize, 4);
	bmp.write((const char*)&dibHorRez, 4);
	bmp.write((const char*)&dibVerRez, 4);
	bmp.write((const char*)&dibNumColoursInPalette, 4);
	bmp.write((const char*)&dibNumImportantColours, 4);

	// Move us to a 4-byte boundary
	// http://www.fileformat.info/format/bmp/egff.htm
	bmp.write((const char*)&reserved, 2);
	// Colour table (Seems we can do without it)

	// Img data:
	for (int y = dibImgHeight - 1; y >= 0; y--) {
		bmp.write((const char*)data + dibImgWidth*4*y, dibImgWidth*4);
	}
	bmp.close();
}

std::string cleanString(const std::string &str) {
	std::string result;
	for (int i = 0; i < str.size(); i++) {
		char c = str.at(i);
		if (isalnum(c)) {
			result += c;
		}
	}
	return result;
}

int parseFile(std::string filename, bool createMovie, bool drawSeparator, unsigned int skipLines) {
	ifstream file(filename);
	if (!file.good()) {
		cout << "Failed to open " << filename << endl;
		return -1;
	}
	cout << "Opened " << filename << endl;
	unsigned int n = 0;
	// Read header TODO
	string line;
	getline(file, line);
	getline(file, line);
	
	vector<vector<string>> lines;
	unsigned int lineNum = 0;
	string prevPs = "";
	while (!file.eof()) {
		stringstream ss;
		getline(file, line);
		if (skipLines > 0) {
			skipLines--;
			continue;
		}
		ss.str(line);
		string ps;
		string delta;
		ss >> ps;
		ss >> delta; // delta
		if (ps == prevPs) {
			lines.pop_back();
			lineNum--;
		}
		prevPs = ps;
		vector<string> internal;
		lines.push_back(internal);
		string item;
		while (!ss.eof()) {
			if (ss >> item) {
				item = cleanString(item); // Get rid of {
				lines[lineNum].push_back(item);
			}
		}
		if (lines.back().size() > 0) {
			lineNum++;
		} else {
			lines.pop_back();
		}
	}
	if (lines.size() < 1) {
		cout << "# lines: " << lines.size() << endl;
		cout << "Can't render an image with no line of input, exiting." << endl;
		return -1;
	}
	unsigned int h = lines[0].size();
	bool oneDim = false;
	if (h == 0) {
		cout << "Height is 0 no image to produce" << endl;
		return -1;
	} else if (h == 1) {
		cout << "This is probably 1D" << endl;
		n = lines[0][0].size();
		oneDim = true;
	} else {
		n = lines[0][1].size(); // width
		if (drawSeparator) {
			h++;
		}
	}
	unsigned int dataSize = lineNum * n * h * 4;
	unsigned char *data = new unsigned char[dataSize];
	unsigned char *dataPtr = data;
	unsigned int bytesEaten = 0;
	for (auto it = lines.begin(); it != lines.end(); ++it) {
		for (auto it2 = it->begin(); it2 != it->end(); ++it2) {
			unsigned int bytes = parseLine(*it2, dataPtr);
			n = bytes;
			dataPtr += bytes * 4;
			bytesEaten += bytes * 4;
		}
		if (!oneDim && drawSeparator) {
			for (unsigned int i = 0; i < n; i++) {
				dataPtr[0] = 255;
				dataPtr[1] = 0;
				dataPtr[2] = 255;
				dataPtr[3] = 0;
				dataPtr += 4;
			}
		}
	}

	cout << "N: " << n << endl;
	cout << "Width: " << n << " Height per box: " << h << " Total Height: " << lineNum * h << endl;
	if (createMovie) {
		for (unsigned int i = 0; i < lineNum; i++) {
			stringstream filename;
			filename << "movie" << i << ".bmp";
			writeBMP(filename.str().c_str(), n, h, data + i * n * h * 4);
		}
	}
	writeBMP("output.bmp", n, lineNum * h, data);

	delete[] data;

	return 0;
}

bool parseUIntParameter(std::string param, unsigned int &result) {
	if (param.size() == 0 || !isdigit(param.at(0))) {
		return false;
	}
	result = atoi(param.c_str());
	return true;
}

void printHelp() {
	cout << "Usage: ./parser -ifilename [-m] [-s]" << endl;
	cout << "\t-m[01] - If non-zero, create a sequence of BMPs suitable for opening as a movie in VirtualDub (default: false)" << endl;
	cout << "\t-s[integer] - Skip this many generations (usefull for skipping load stages)" << endl;
	cout << "\t-d[01] - If non-zero, draw a separating line between generations in 2D (default: true)" << endl;
}

int main(int argc, char** argv) {
	unsigned int skipLines = 0;
	std::string inputFileName = "";
	bool createMovie = false;
	bool drawSeparator = true;
	if (argc == 1) {
		printHelp();
		return 0;
	}
	for (int i = 1; i < argc; i++) {
		std::string arg(argv[i]);
		if (arg.at(0) == '-' && arg.size() > 1) {
			char param = tolower(arg.at(1));
			arg = arg.substr(2, string::npos);
			unsigned int result = 0;
			switch (param) {
				case 'm':
					if (parseUIntParameter(arg, result)) {
						createMovie = result;
					}
					break;
				case 's':
					if (parseUIntParameter(arg, result)) {
						skipLines = result;
					}
					break;
				case 'd':
					if (parseUIntParameter(arg, result)) {
						drawSeparator = result;
					}
					break;
				case 'i':
					inputFileName = arg;
					break;
				default: {
					printHelp();
					return 0;
				}
				break;
			}
		}
	}
	if (inputFileName == "") {
		printHelp();
		return 0;
	}
	return parseFile(inputFileName, createMovie, drawSeparator, skipLines);
}
